@@ -2,14 +2,18 @@
 #
 # Author: Thamme Gowda [tg (at) isi (dot) edu] 
 # Created: 4/8/20
-from mtdata import log, cached_index_file, __version__
-from mtdata.entry import Entry, Paper
-from typing import List, Optional
-from pathlib import Path
+import collections
 import pickle
+from pathlib import Path
+from typing import List, Dict, Union
+
 import portalocker
 from pybtex.database import parse_file as parse_bib_file
 
+from mtdata import log, cached_index_file, __version__
+from mtdata.entry import Entry, DatasetId
+from mtdata.iso.bcp47 import bcp47, BCP47Tag
+
 REFS_FILE = Path(__file__).parent / 'refs.bib'
 
 
@@ -18,8 +22,8 @@ class Index:
     obj = None   # singleton object
 
     def __init__(self):
-        self.entries = {}  # unique
-        self.papers = {}  # unique
+        self.entries: Dict[DatasetId, Entry] = {}  # unique dids
+        self.papers = {}                            # unique
         self.ref_db = ReferenceDb()
         self.version = __version__
 
@@ -49,14 +53,11 @@ class Index:
             cls.obj = obj
         return cls.obj
 
-
     def load_all(self):
-        from mtdata.index import (statmt, paracrawl, tilde, literature, joshua_indian,
-                                  unitednations, wikimatrix, other, neulab_tedtalks, elrc_share,
-                                  ai4bharat, eu)
+        from mtdata.index import (
+            statmt, paracrawl, tilde, joshua_indian, unitednations, wikimatrix, other, neulab_tedtalks,
+            elrc_share, ai4bharat, eu, linguatools)
         from mtdata.index.opus import opus_index, jw300, opus100
-
-        counts = {}
         subsets = [
             ('Statmt.org', statmt.load),
             ('Paracrawl', paracrawl.load),
@@ -71,17 +72,19 @@ class Index:
             ('Neulab_TEDTalksv1', neulab_tedtalks.load_all),
             ('ELRC-SHARE', elrc_share.load_all),
             ('AI4Bharat', ai4bharat.load_all),
-            ('EU', eu.load_all)
+            ('EU', eu.load_all),
+            ('LinguaTools', linguatools.load_all)
         ]
         for name, loader in subsets:
-            n = len(self)
             loader(self)
-            counts[name] = len(self) - n
+
+        counts = collections.defaultdict(int)
+        for e in self.entries.values():
+            counts[e.did.group] += 1
         items = list(sorted(counts.items(), key=lambda x: x[1], reverse=True))
         items += [('Total', len(self))]
-        counts = '  '.join([f'{n}:{c:,}' for n, c in items])
-        log.info(f"Index status: {counts}")
-        literature.load(self)
+        counts = '\n'.join([f'| {n} | {c:,}|' for n, c in items])
+        log.info(f"Index status:\n{counts}")
 
     @property
     def n_entries(self) -> int:
@@ -89,24 +92,23 @@ class Index:
 
     def add_entry(self, entry: Entry):
         assert isinstance(entry, Entry)
-        key = (entry.name, entry.langs)
+        key = entry.did
         assert key not in self.entries, f'{key} is a duplicate'
         self.entries[key] = entry
 
-    def add_paper(self, paper: Paper):
-        assert isinstance(paper, Paper)
-        assert paper.name not in self.papers, f'{paper.name} is a duplicate'
-        self.papers[paper.name] = paper
-
     def get_entries(self):
         return self.entries.values()
 
     def get_papers(self):
         return self.papers.values()
 
-    def contains_entry(self, name, langs):
-        key = (name, langs)
-        return key in self.entries
+    def __contains__(self, item):
+        assert isinstance(item, DatasetId)
+        return item in self.entries
+
+    def __getitem__(self, item):
+        assert isinstance(item, DatasetId)
+        return self.entries[item]
 
     def contains_paper(self, name):
         return name in self.papers
@@ -114,6 +116,7 @@ class Index:
     def get_entry(self, name, langs):
         assert isinstance(name, str)
         assert isinstance(langs, tuple)
+        langs = tuple(lang if isinstance(lang, BCP47Tag) else bcp47(lang) for lang in langs)
         key = (name, langs)
         rev_key = (name, tuple(reversed(langs)))
         if key not in self.entries and rev_key in self.entries:
@@ -126,6 +129,7 @@ class Index:
     def __len__(self):
         return len(self.entries)
 
+
 class ReferenceDb:
 
     _instance = None  # singleton instance
@@ -154,26 +158,43 @@ class ReferenceDb:
         return self.db.entries.keys()
 
 
-def get_entries(langs=None, names=None, not_names=None) -> List[Entry]:
+def is_compatible(lang1: Union[str, BCP47Tag], lang2: Union[str, BCP47Tag]):
+    lang1 = lang1 if isinstance(lang1, BCP47Tag) else bcp47(lang1)
+    lang2 = lang2 if isinstance(lang2, BCP47Tag) else bcp47(lang2)
+    return lang1.is_compatible(lang2)
+
+
+def bitext_lang_match(pair1, pair2, fuzzy_match=False) -> bool:
+    x1, y1 = sorted(pair1)
+    x2, y2 = sorted(pair2)
+    if fuzzy_match:
+        return is_compatible(x1, x2) and is_compatible(y1, y2)
+    else:
+        return x1 == x2 and y1 == y1
+
+
+def get_entries(langs=None, names=None, not_names=None, fuzzy_match=False) -> List[Entry]:
     """
     :param langs: language pairs  to select eg ('en', 'de')
     :param names:  names to select
     :param not_names:  names to exclude
+    :param fuzzy_match
     :return: list of dataset entries that match the criteria
     """
+    # TODO: our index has grown too big; improve search with fuzzy matches
     select = list(INDEX.get_entries())
     if names:
-        if not isinstance(names, set):
-            names = set(names)
-        select = [e for e in select if e.name in names]
+        names = set(n.lower() for n in names)
+        select = [e for e in select if e.did.name in names]
     if langs:
         assert len(langs) == 2
         langs = sorted(langs)
-        select = [e for e in select if langs == sorted(e.langs)]
+        select = [e for e in select if bitext_lang_match(langs, e.did.langs, fuzzy_match=fuzzy_match)]
     if not_names:
         if not isinstance(not_names, set):
             not_names = set(not_names)
-        select = [e for e in select if e.name not in not_names]
+        select = [e for e in select if e.did.name not in not_names]
     return select
 
+
 INDEX: Index = Index.get_instance()
