@@ -2,18 +2,20 @@
 #
 # Author: Thamme Gowda [tg (at) isi (dot) edu] 
 # Created: 4/8/20
-from mtdata.index import Entry, Index
+from mtdata.index import Entry, Index, DatasetId, bcp47
 import itertools
 
 
 def load(index: Index):
+    group_id = 'Statmt'
     WMT13_CCRAWL = "http://www.statmt.org/wmt13/training-parallel-commoncrawl.tgz"
     WMT14_CITE = index.ref_db.get_bibtex('ws-2014-statistical')
     for l1 in ['de', 'cs', 'fr', 'ru', 'es']:
         l2 = 'en'
         f1 = f'commoncrawl.{l1}-en.{l1}'
         f2 = f'commoncrawl.{l1}-en.en'
-        index.add_entry(Entry(langs=(l1, l2), name=f'wmt13_commoncrawl', url=WMT13_CCRAWL,
+        data_id = DatasetId(group=group_id, name='commoncrawl_wmt13', version='1', langs=(l1, l2))
+        index.add_entry(Entry(did=data_id, url=WMT13_CCRAWL,
                               filename='wmt13_parallel_commoncrawl.tgz',
                               in_paths=[f1, f2], in_ext='txt', cite=WMT14_CITE))
 
@@ -22,8 +24,8 @@ def load(index: Index):
         l2 = 'en'
         f1 = f'training/europarl-v7.{l1}-{l2}.{l1}'
         f2 = f'training/europarl-v7.{l1}-{l2}.{l2}'
-        index.add_entry(Entry(langs=(l1, l2), name=f'wmt13_europarl_v7',
-                              url="http://www.statmt.org/wmt13/training-parallel-europarl-v7.tgz",
+        data_id = DatasetId(group=group_id, name='europarl_wmt13', version='7', langs=(l1, l2))
+        index.add_entry(Entry(did=data_id, url="http://www.statmt.org/wmt13/training-parallel-europarl-v7.tgz",
                               filename="wmt13_europarl_v7.tgz",
                               in_paths=[f1, f2], in_ext='txt', cite=WMT14_CITE))
 
@@ -32,7 +34,8 @@ def load(index: Index):
         l2 = 'en'
         f1 = f'training-parallel-nc-v13/news-commentary-v13.{l1}-{l2}.{l1}'
         f2 = f'training-parallel-nc-v13/news-commentary-v13.{l1}-{l2}.{l2}'
-        index.add_entry(Entry(langs=(l1, l2), name=f'wmt18_news_commentary_v13',
+        data_id = DatasetId(group=group_id, name='news_commentary_wmt18', version='13', langs=(l1, l2))
+        index.add_entry(Entry(did=data_id,
                               url="http://data.statmt.org/wmt18/translation-task/training-parallel-nc-v13.tgz",
                               filename="wmt18_news_commentary_v13.tgz",
                               in_paths=[f1, f2], in_ext='txt', cite=WMT14_CITE))
@@ -42,8 +45,8 @@ def load(index: Index):
     cite = index.ref_db.get_bibtex('koehn2005europarl')
     for pair in ['de en', 'cs en', 'cs pl', 'es pt', 'fi en', 'lt en']:
         l1, l2 = pair.split()
-        index.add_entry(
-            Entry(langs=(l1, l2), name='europarl_v9', url=EUROPARL_v9 % (l1, l2), cite=cite))
+        index.add_entry(Entry(did=DatasetId(group=group_id, name='europarl', version='9', langs=(l1, l2)),
+                              url=EUROPARL_v9 % (l1, l2), cite=cite))
 
     # === Europarl V7 corpus
     EUROPARL_v7 = 'http://www.statmt.org/europarl/v7/%s-%s.tgz'
@@ -52,15 +55,16 @@ def load(index: Index):
         l2 = 'en'
         src = f'europarl-v7.{l1}-{l2}.{l1}'
         ref = f'europarl-v7.{l1}-{l2}.{l2}'
-        index.add_entry(Entry(langs=(l1, l2), name='europarl_v7', in_paths=[src, ref],
+        index.add_entry(Entry(
+            did=DatasetId(group=group_id, name='europarl', version='7', langs=(l1, l2)), in_paths=[src, ref],
                               url=EUROPARL_v7 % (l1, l2), in_ext='txt', cite=cite))
 
     # === Digital Corpus of European Parliament
-    index.add_entry(Entry(langs=('lv', 'en'), name='wmt17_dcep_v1',
-                          in_paths=['*/*.lv', f'*/*.en'], cite=cite,
+    index.add_entry(Entry(did=DatasetId(group=group_id, name='dcep_wmt17', version='1', langs=(l1, l2)),
+                          in_paths=['*/*.lv', f'*/*.en'], cite=cite, in_ext='txt',
                           url='http://data.statmt.org/wmt17/translation-task/dcep.lv-en.v1.tgz'))
-    index.add_entry(Entry(langs=('lv', 'en'), name='wmt17_books_v1',
-                          in_paths=['*/*.lv', f'*/*.en'], cite=cite,
+    index.add_entry(Entry(did=DatasetId(group=group_id, name='books_wmt17', version='1', langs=(l1, l2)),
+                          in_paths=['*/*.lv', f'*/*.en'], cite=cite, in_ext='txt',
                           url='http://data.statmt.org/wmt17/translation-task/books.lv-en.v1.tgz'))
 
     # === News Commentary v14
@@ -78,9 +82,27 @@ def load(index: Index):
                  'id zh', 'it kk', 'it nl', 'it pt', 'it ru', 'it zh', 'ja ru', 'ja zh', 'kk nl',
                  'kk pt', 'kk ru', 'kk zh', 'nl pt', 'nl ru', 'nl zh', 'pt ru', 'pt zh', 'ru zh']:
         l1, l2 = pair.split()
-        index.add_entry(
-            Entry(langs=(l1, l2), name='news_commentary_v14', url=NEWSCOM_v14 % (l1, l2),
-                  cite=cite))
+        index.add_entry(Entry(
+            did=DatasetId(group=group_id, name='news_commentary', version='14', langs=(l1, l2)),
+            url=NEWSCOM_v14 % (l1, l2), cite=cite))
+
+    for v in [15, 16]:
+        cite = index.ref_db.get_bibtex('barrault-etal-2020-findings')
+        url = f"http://data.statmt.org/news-commentary/v{v}/training/news-commentary-v{v}.%s-%s.tsv.gz"
+        for pair in ['ar cs', 'ar de', 'ar en', 'ar es', 'ar fr', 'ar hi', 'ar id', 'ar it', 'ar ja', 'ar kk', 'ar nl',
+                     'ar pt', 'ar ru', 'ar zh', 'cs de', 'cs en', 'cs es', 'cs fr', 'cs hi', 'cs id', 'cs it', 'cs ja',
+                     'cs kk', 'cs nl', 'cs pt', 'cs ru', 'cs zh', 'de en', 'de es', 'de fr', 'de hi', 'de id', 'de it',
+                     'de ja', 'de kk', 'de nl', 'de pt', 'de ru', 'de zh', 'en es', 'en fr', 'en hi', 'en id', 'en it',
+                     'en ja', 'en kk', 'en nl', 'en pt', 'en ru', 'en zh', 'es fr', 'es hi', 'es id', 'es it', 'es ja',
+                     'es kk', 'es nl', 'es pt', 'es ru', 'es zh', 'fr hi', 'fr id', 'fr it', 'fr ja', 'fr kk', 'fr nl',
+                     'fr pt', 'fr ru', 'fr zh', 'hi id', 'hi it', 'hi nl', 'hi pt', 'hi ru', 'hi zh', 'id it', 'id ja',
+                     'id kk', 'id nl', 'id pt', 'id ru', 'id zh', 'it kk', 'it nl', 'it pt', 'it ru', 'it zh', 'ja pt',
+                     'ja ru', 'ja zh', 'kk nl', 'kk pt', 'kk ru', 'kk zh', 'nl pt', 'nl ru', 'nl zh', 'pt ru', 'pt zh',
+                     'ru zh']:
+            l1, l2 = pair.split()
+            index.add_entry(Entry(did=DatasetId(group=group_id, name='news_commentary', version=f'{v}', langs=(l1, l2)),
+                url=url % (l1, l2), cite=cite))
+
 
     # ===== Wiki Titles V1
     WIKI_TITLES_v1 = 'http://data.statmt.org/wikititles/v1/wikititles-v1.%s-%s.tsv.gz'
@@ -88,16 +110,16 @@ def load(index: Index):
     for pair in ['cs en', 'cs pl', 'de en', 'es pt', 'fi en', 'gu en', 'hi ne', 'kk en', 'lt en',
                  'ru en', 'zh en']:
         l1, l2 = pair.split()
-        index.add_entry(Entry(langs=(l1, l2), name='wiki_titles_v1', url=WIKI_TITLES_v1 % (l1, l2),
-                              cite=cite))
+        index.add_entry(Entry(did=DatasetId(group=group_id, name='wiki_titles', version='1', langs=(l1, l2)),
+                              url=WIKI_TITLES_v1 % (l1, l2), cite=cite))
 
     # ===== Wiki Titles V2
     WIKI_TITLES_v2 = 'http://data.statmt.org/wikititles/v2/wikititles-v2.%s-%s.tsv.gz'
     for pair in ['ca es', 'cs en', 'de en', 'de fr', 'es pt', 'iu en', 'ja en', 'pl en', 'ps en',
                  'ru en', 'ta en', 'zh en']:
         l1, l2 = pair.split()
-        index.add_entry(Entry(langs=(l1, l2), name='wiki_titles_v2', url=WIKI_TITLES_v2 % (l1, l2),
-                              cite=cite))
+        index.add_entry(Entry(did=DatasetId(group=group_id, name='wiki_titles', version='2', langs=(l1, l2)),
+                              url=WIKI_TITLES_v2 % (l1, l2), cite=cite))
 
     # ==== WMT  Dev and Tests
     wmt_sets = {
@@ -127,11 +149,13 @@ def load(index: Index):
                         ('en', 'ja'), ('ja', 'en'), ('en', 'pl')]
     }
     for set_name, pairs in wmt_sets.items():
+        sub_name, year = set_name[:-4], set_name[-4:]
         for l1, l2 in pairs:
             src = f'dev/{set_name}-{l1}{l2}-src.{l1}.sgm'
             ref = f'dev/{set_name}-{l1}{l2}-ref.{l2}.sgm'
-            name = f'{set_name}_{l1}{l2}'
-            index.add_entry(Entry((l1, l2), name=name, filename='wmt20dev.tgz', in_paths=[src, ref],
+            name = f'{sub_name}_{l1}{l2}'
+            index.add_entry(Entry(did=DatasetId(group=group_id, name=name, version=year, langs=(l1, l2)),
+                                  filename='wmt20dev.tgz', in_paths=[src, ref], in_ext='sgm',
                                   url='http://data.statmt.org/wmt20/translation-task/dev.tgz',
                                   cite=cite))
 
@@ -144,12 +168,12 @@ def load(index: Index):
         '2013': ['en', 'cs', 'de', 'es', 'fr', 'ru'],
     }
     for year, langs in wmt_sets.items():
+        name = 'newstest'
         for l1, l2 in itertools.combinations(langs, 2):
-            name = f'newstest{year}'
-            f1 = f'dev/{name}.{l1}'
-            f2 = f'dev/{name}.{l2}'
-            index.add_entry(Entry((l1, l2), name=name, filename='wmt20dev.tgz', in_paths=[f1, f2],
-                                  in_ext='txt', cite=cite,
+            f1 = f'dev/{name}{year}.{l1}'
+            f2 = f'dev/{name}{year}.{l2}'
+            index.add_entry(Entry(did=DatasetId(group=group_id, name=name, version=year, langs=(l1, l2)),
+                                    filename='wmt20dev.tgz', in_paths=[f1, f2], in_ext='txt', cite=cite,
                                   url='http://data.statmt.org/wmt20/translation-task/dev.tgz'))
 
     for l1, l2 in [('ps', 'en'), ('km', 'en')]:
@@ -157,9 +181,9 @@ def load(index: Index):
             src = f'dev/{set_name}.{l1}-{l2}.{l1}'
             ref = f'dev/{set_name}.{l1}-{l2}.{l2}'
             name = f'{set_name.replace(".", "_")}_{l1}{l2}'
-            index.add_entry(Entry((l1, l2), name=name, filename='wmt20dev.tgz', in_paths=[src, ref],
-                                  url='http://data.statmt.org/wmt20/translation-task/dev.tgz',
-                                  in_ext='txt', cite=cite))
+            index.add_entry(Entry(did=DatasetId(group=group_id, name=name, version='1', langs=(l1, l2)),
+                                 filename='wmt20dev.tgz', in_paths=[src, ref], in_ext='txt', cite=cite,
+                                  url='http://data.statmt.org/wmt20/translation-task/dev.tgz'))
 
     #### WMT 20 Tests
     url = "http://data.statmt.org/wmt20/translation-task/test.tgz"
@@ -169,37 +193,36 @@ def load(index: Index):
              "enru", "enta", "enzh", "frde", "iuen", "jaen", "kmen", "plen", "psen", "ruen",
              "taen", "zhen"],
         "B": ["deen", "ende", "enzh", "ruen", "zhen"]}.items():
+        year = "2020"
+        name = f'newstest{_pref}'
         for pair in pairs:
             l1, l2 = pair[:2], pair[2:]
-            name = f'newstest{_pref}2020'
-            f1 = f'sgm/{name}-{pair}-src.{l1}.sgm'
-            f2 = f'sgm/{name}-{pair}-ref.{l2}.sgm'
-            name = f'{name}_{pair}'  # name cant have -, so have an _ instead
-            index.add_entry(Entry((l1, l2), name=name, filename='wmt20tests.tgz', in_paths=[f1, f2],
-                                  in_ext='sgm', cite=wmt20_cite, url=url))
+            f1 = f'sgm/{name}{year}-{pair}-src.{l1}.sgm'
+            f2 = f'sgm/{name}{year}-{pair}-ref.{l2}.sgm'
+            index.add_entry(Entry(did=DatasetId(group=group_id, name=f'{name}_{pair}'.lower(), version=year, langs=(l1, l2)),
+                filename='wmt20tests.tgz', in_paths=[f1, f2], in_ext='sgm', cite=wmt20_cite, url=url))
 
     # WMT 21 Dev
     url = "http://data.statmt.org/wmt21/translation-task/dev.tgz"
     pairs = "en-ha en-is is-en ha-en".split()
     for pair in pairs:
         l1, l2 = pair.split('-')
-        name = f'newsdev2021_{l1}{l2}'
         in_path = f'dev/xml/newsdev2021.{l1}-{l2}.xml'
-        ent = Entry((l1, l2), name=name, filename='wmt21dev.tgz', in_paths=[in_path],
-              in_ext='wmt21xml', cite=wmt20_cite, url=url)
+        ent = Entry(did=DatasetId(group=group_id, name=f'newsdev_{l1}{l2}', version='2021', langs=(l1, l2)),
+                    filename='wmt21dev.tgz', in_paths=[in_path], in_ext='wmt21xml', cite=wmt20_cite, url=url)
         index.add_entry(ent)
 
 
     # ==== TED Talks 2.0 ar-en
-    index.add_entry(Entry(('en', 'ar'), 'tedtalks_v2_clean', ext='tsv.xz',
-                          url='http://data.statmt.org/ted-talks/en-ar.v2.aligned.clean.xz'))
+    index.add_entry(Entry(did=DatasetId(group=group_id, name='tedtalks', version='2_clean', langs=('en', 'ar')),
+                         ext='tsv.xz', url='http://data.statmt.org/ted-talks/en-ar.v2.aligned.clean.xz'))
 
     # ==== Europarl v10
     EP_v10 = "http://www.statmt.org/europarl/v10/training/europarl-v10.%s-%s.tsv.gz"
     for pair in ['cs en', 'cs pl', 'de en', 'de fr', 'es pt', 'fi en', 'fr en', 'lt en', 'pl en']:
         l1, l2 = pair.split()
-        index.add_entry(
-            Entry(langs=(l1, l2), name='europarl_v10', url=EP_v10 % (l1, l2), cite=wmt20_cite))
+        index.add_entry(Entry(did=DatasetId(group=group_id, name=f'europarl', version='10', langs=(l1, l2)),
+                url=EP_v10 % (l1, l2), cite=wmt20_cite))
 
     # ==== PMIndia V1
     PMINDIA_v1 = "http://data.statmt.org/pmindia/v1/parallel/pmindia.v1.%s-%s.tsv"
@@ -208,11 +231,11 @@ def load(index: Index):
                  "pa en", "ta en", "te en", "ur en"]:
         l1, l2 = pair.split()
         # Note: listed as xx-en in URL but actually en-xx in the tsv; and its not compressed!
-        index.add_entry(
-            Entry(langs=(l2, l1), name='pmindia_v1', url=PMINDIA_v1 % (l1, l2), cite=cite))
+        index.add_entry(Entry(did=DatasetId(group=group_id, name=f'pmindia', version='1', langs=(l2, l1)),
+                              url=PMINDIA_v1 % (l1, l2), cite=cite))
 
     # Pashto - English  pseudo parallel dataset for alignment
-    index.add_entry(Entry(langs=('en', 'ps'), name='wmt20_enps_aligntask',
+    index.add_entry(Entry(did=DatasetId(group=group_id, name=f'wmt20_enps_aligntask', version='1', langs=('en', 'ps')),
                           url='http://data.statmt.org/wmt20/translation-task/ps-km/wmt20-sent.en-ps.xz',
                           cite=wmt20_cite, ext='tsv.xz'))
 
@@ -223,14 +246,14 @@ def load(index: Index):
         en = f'ps-parallel/{name}.en'
         url = 'http://data.statmt.org/wmt20/translation-task/ps-km/ps-parallel.tgz'
         name = name.replace('.en-ps', '').replace('.', '_').replace('-', '_').lower()
-        entry = Entry(langs=('ps', 'en'), name=name, url=url, cite=wmt20_cite, in_paths=[ps, en],
-                      filename='wmt20-psen-parallel.tgz', in_ext='txt')
+        entry = Entry(did=DatasetId(group=group_id, name=name, version='1', langs=('ps', 'en')), url=url,
+                      cite=wmt20_cite, in_paths=[ps, en], filename='wmt20-psen-parallel.tgz', in_ext='txt')
         index.add_entry(entry)
 
     for l2 in ['ps', 'km']:
         url = f"http://data.statmt.org/wmt20/translation-task/ps-km/wmt20-sent.en-{l2}.xz"
-        entry = Entry(langs=('en', l2), name='paracrawl_v5_1', url=url, cite=wmt20_cite,
-                      ext='tsv.xz', cols=(0, 1))
+        entry = Entry(did=DatasetId(group=group_id, name='paracrawl', version='5.1', langs=('en', l2)),
+                    url=url, cite=wmt20_cite, ext='tsv.xz', cols=(0, 1))
         index.add_entry(entry)
 
     ccalign_cite = index.ref_db.get_bibtex('chaudhary-EtAl:2019:WMT')
@@ -238,20 +261,17 @@ def load(index: Index):
     tgts='es_XX et_EE fa_IR ff_NG fi_FI fr_XX gu_IN ha_NG he_IL hi_IN hr_HR ht_HT hu_HU hy_AM id_ID ig_NG is_IS it_IT ja_XX jv_ID ka_GE kg_AO kk_KZ km_KH kn_IN ko_KR ku_TR ky_KG lg_UG ln_CD lo_LA lt_LT lv_LV mg_MG mi_NZ mk_MK ml_IN mn_MN mr_IN ms_MY mt_MT my_MM ne_NP nl_XX no_XX ns_ZA ny_MW om_KE or_IN pa_IN pl_PL ps_AF pt_XX qa_MM qd_MM ro_RO ru_RU si_LK sk_SK sl_SI sn_ZW so_SO sq_AL sr_RS ss_SZ st_ZA su_ID sv_SE sw_KE sz_PL ta_IN te_IN tg_TJ th_TH ti_ET tl_XX tn_BW tr_TR ts_ZA tz_MA uk_UA ur_PK ve_ZA vi_VN wo_SN xh_ZA yo_NG zh_CN zh_TW zu_ZA zz_TR'.split()
     srcs = 'af_ZA ak_GH am_ET ar_AR as_IN ay_BO az_AZ az_IR be_BY bg_BG bm_ML bn_IN br_FR bs_BA ca_ES cb_IQ cs_CZ cx_PH cy_GB da_DK de_DE el_GR'.split()
     pairs = [('en_XX', tgt) for tgt in tgts] + [(src, 'en_XX') for src in srcs]
-    dont_know = {'qa', 'qd'}
-     # Cant find them in ISO 639-1:  https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes
-     #                and lingo http://www.lingoes.net/en/translator/langcode.htm
-     #               and web-info https://wp-info.org/tools/languagecodes.php
-    unsupported = {'zh_TW', 'az_IR'}
+    dont_know = {'qa', 'qd'}   # looks like some Myanmar languages, but not sure which one.
+    # Cant find them in ISO 639-1:  https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes
+    #                and lingo http://www.lingoes.net/en/translator/langcode.htm
+    #               and web-info https://wp-info.org/tools/languagecodes.php
+    #unsupported = {'zh_TW', 'az_IR'}
     # country locales are not supported; they create conflicts. keeping large ones instead
     for src, tgt in pairs:
-        if src in unsupported or tgt in unsupported:
-            continue
-        l1, l2 = src.split('_')[0], tgt.split('_')[0]
-        if l1 in dont_know or l2 in dont_know:
-            # I dont know what language these are
+        # l1, l2 = src.split('_')[0], tgt.split('_')[0]
+        if src[:2] in dont_know or tgt[:2] in dont_know:   # I dont know what language these are
             continue
         url = CC_ALIGNED.format(src=src, tgt=tgt)
-        entry = Entry(langs=(l1, l2), name='cc_aligned', url=url, cite=ccalign_cite,
-                      ext='tsv.xz', cols=(0, 1))
+        entry = Entry(did=DatasetId(group=group_id, name='ccaligned', version='1', langs=(src, tgt)), url=url,
+                      cite=ccalign_cite, ext='tsv.xz', cols=(0, 1))
         index.add_entry(entry)
