@@ -6,7 +6,8 @@
 from typing import Optional, Union, Tuple, List
 from dataclasses import dataclass
 from pathlib import Path
-from mtdata import log
+from mtdata import log, pbar_man
+from mtdata.entry import Entry
 from itertools import zip_longest
 
 from mtdata.utils import IO
@@ -32,7 +33,6 @@ def detect_extension(name: Union[str, Path]):
 @dataclass
 class Parser:
     paths: Union[Path, List[Path]]
-    langs: Tuple[str, str]
     ext: Optional[str] = None
     ent: Optional['Entry'] = None
 
@@ -44,7 +44,7 @@ class Parser:
 
         if not self.ext:
             exts = [detect_extension(p.name) for p in self.paths]
-            if len(exts) == 2 and set(exts) == set(self.langs):
+            if len(exts) == 2:
                 log.warning(f"Treating {' .'.join(exts)} as plain text. To override: in_ext=<ext>")
                 exts = ['txt']  # treat that as plain text
             assert len(set(exts)) == 1, f'Expected a type of exts, but found: {exts}'
@@ -56,9 +56,12 @@ class Parser:
     def read_segs(self):
         readers = []
         if self.ext == 'opus_xces':
+            preprocessing = 'xml'
+            if "/raw/" in self.ent.in_paths[0]:
+                preprocessing = 'raw'
             align, lang1_dir, lang2_dir = self.paths
             from mtdata.opus_xces import OpusXcesParser
-            reader = OpusXcesParser.read(align, lang1_dir, lang2_dir)
+            reader = OpusXcesParser.read(align, lang1_dir, lang2_dir, preprocessing=preprocessing)
             readers.append(reader)
         else:
             for p in self.paths:
@@ -71,7 +74,7 @@ class Parser:
                     readers.append(self.read_plain(p))
                 elif 'tmx' in self.ext:
                     from mtdata.tmx import read_tmx
-                    readers.append(read_tmx(path=p, langs=self.langs))
+                    readers.append(read_tmx(path=p, langs=self.ent.did.langs))
                 elif 'sgm' in self.ext:
                     from mtdata.sgm import read_sgm
                     readers.append(read_sgm(p))
@@ -82,17 +85,22 @@ class Parser:
                     raise Exception(f'Not supported {self.ext} : {p}')
 
         if len(readers) == 1:
-            yield from readers[0]
+            data = readers[0]
         elif self.ext == 'tmx' or self.ext == 'tsv':
-            for reader in readers:
-                yield from reader
+            data = (rec for reader in readers for rec in reader)  # flatten all readers
         elif len(readers) == 2:
-            for seg1, seg2 in zip_longest(*readers):
-                if seg1 is None or seg2 is None:
-                    raise Exception(f'{self.paths} have unequal number of segments')
-                yield seg1, seg2
+            def _zip_n_check():
+                for seg1, seg2 in zip_longest(*readers):
+                    if seg1 is None or seg2 is None:
+                        raise Exception(f'{self.paths} have unequal number of segments')
+                    yield seg1, seg2
+            data = _zip_n_check()
         else:
             raise Exception("This is an error")
+        with pbar_man.counter(color='green', unit='seg', leave=False, desc=f"Reading {self.ent.did}") as pbar:
+            for rec in data:
+                yield rec
+                pbar.update()
 
     def read_plain(self, path):
         with IO.reader(path) as stream:
